
package
{
	import flash.display.*;
	import flash.events.*;
	import flash.geom.*;
	import flash.media.*;
	import flash.utils.getTimer;
	import gs.*;

	public class Invader extends Sprite
	{
		public var game: Game;
		
		[Embed(source="images/invader.png")]
		public static var invaderGfx: Class;
		
		[Embed(source="images/invader2.png")]
		public static var invader2Gfx: Class;
		
		[Embed(source="images/invader2.png")]
		public static var invader3Gfx: Class;
		
		[Embed(source="audio/hit.mp3")]
		public static var hitSfx: Class;
		
		private var hitSound: Sound;
		
		private var image: Bitmap;
		
		private var vx: Number;
		private var speed: Number = 2;
		
		// used when dead
		private var vy: Number;
		private var vRot: Number;
		private var gravity: Number;
		
		private const maxComponents: int = 46;
		private var components: int = maxComponents;
		
		private var dead: Boolean = false;
		
		public function Invader (_game: Game)
		{
			game = _game;
			
			x = 200;
			y = 80;
			
			vx = speed;
			
			image = new invaderGfx();
			
			image.x = -image.width/2;
			image.y = -image.height/2;
			
			addChild(image);
			
			hitSound = new hitSfx();
			
			// used when dead
			vy = 0;
			vRot = Math.random() - 0.5;
			gravity = Math.random() * 0.05 + 0.05;
		}
		
		public function update (): void
		{
			if (dead)
			{
				vy += gravity;
				
				//x += vx;
				y += vy;
				rotation += vRot;
				
				addDebris(x, y, Math.random() * 4 - 2, -Math.random()*2 - 2);
				
				return;
			}
			
			if (x - image.width/2 < 10)
			{
				vx = speed;
				y += 10;
			}
			
			if (x + image.width/2 > 640 - 10)
			{
				vx = -speed;
				y += 10;
			}
			
			x += vx;
		}
		
		public function check (bullet: Bullet): void
		{
			var point: Point = new Point(bullet.x, bullet.y);
			
			point = image.globalToLocal(point);
			
			var colour: uint = image.bitmapData.getPixel32(point.x, point.y);
			
			var r: uint = (colour >> 16) & 0xFF;

			var hit: Boolean = (r > 0);
			
			if (hit)
			{
				var ix: int = point.x / 16;
				var iy: int = point.y / 16;
				
				var rect: Rectangle = new Rectangle(ix*16, iy*16, 16, 16);
				
				image.bitmapData.fillRect(rect, 0xFF000000);
				
				bullet.y = -50;
				
				point.x = rect.x + 8;
				point.y = rect.y + 8;
				
				point = image.localToGlobal(point);
				
				addDebris(point.x, point.y, vx);
				
				hitSound.play();
				
				speed += 0.075;
				
				components -= 1;
				
				if (components == 0)
				{
					dead = true;
				}
			}
		}
		
		public function addDebris (x: Number, y:Number, vx: Number = 0, vy: Number = 0): void {
			var particle: DebrisParticle = new DebrisParticle(x, y, vx, vy);
			
			particle.graphics.beginFill(0x111111);
			particle.graphics.drawRect(-8, -8, 16, 16);
			particle.graphics.endFill();
			
			particle.graphics.beginFill(0xc12121);
			particle.graphics.drawRect(-7, -7, 14, 14);
			particle.graphics.endFill();
			
			game.addDebris(particle);
		}
	}

}


